@@ -2,7 +2,9 @@
 //
 // Copyright IBM Corp. 2023, 2024
 
-use crate::{confidential::Confidential, error::Result, Error};
+use std::{convert::TryInto, fmt::Display, ops::Range};
+
+use enum_dispatch::enum_dispatch;
 use openssl::{
     derive::Deriver,
     ec::{EcGroup, EcKey},
@@ -14,33 +16,98 @@ use openssl::{
     rand::rand_bytes,
     rsa::Padding,
     sign::{Signer, Verifier},
-    symm::{decrypt_aead, encrypt_aead, Cipher},
+    symm::{decrypt_aead as openssl_decrypt_aead, encrypt_aead as openssl_encrypt_aead, Cipher},
 };
-use std::{convert::TryInto, ops::Range};
+use pv_core::request::Confidential;
 
-/// An AES256-key that will purge itself out of the memory when going out of scope
-pub type Aes256Key = Confidential<[u8; 32]>;
-pub(crate) const AES_256_GCM_TAG_SIZE: usize = 16;
+use crate::{error::Result, Error};
+
+/// An AES256-GCM key that will purge itself out of the memory when going out of scope
+pub type Aes256GcmKey = Confidential<[u8; SymKeyType::AES_256_GCM_KEY_LEN]>;
+/// An AES256-XTS key that will purge itself out of the memory when going out of scope
+pub type Aes256XtsKey = Confidential<[u8; SymKeyType::AES_256_XTS_KEY_LEN]>;
+
+/// SHA-512 digest length (in bytes)
+pub const SHA_512_HASH_LEN: usize = 64;
 
 #[allow(dead_code)]
-pub(crate) const SHA_256_HASH_SIZE: u32 = 32;
+pub(crate) const SHA_256_HASH_LEN: u32 = 32;
 #[allow(dead_code)]
-pub(crate) type Sha256Hash = [u8; SHA_256_HASH_SIZE as usize];
+pub(crate) type Sha256Hash = [u8; SHA_256_HASH_LEN as usize];
 
 /// Types of symmetric keys, to specify during construction.
 #[non_exhaustive]
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum SymKeyType {
-    /// AES 256 key (32 bytes)
-    Aes256,
+    /// AES 256 GCM key (32 bytes)
+    Aes256Gcm,
+    /// AES 256 XTS key (64 bytes)
+    Aes256Xts,
 }
 
+impl SymKeyType {
+    #[deprecated]
+    #[allow(non_upper_case_globals)]
+    /// AES 256 GCM key (32 bytes)
+    pub const Aes256: Self = Self::Aes256Gcm;
+    /// AES256-GCM key length (in bytes)
+    pub const AES_256_GCM_KEY_LEN: usize = 32;
+    /// AES256-GCM IV length (in bytes)
+    pub const AES_256_GCM_IV_LEN: usize = 12;
+    /// AES256-GCM tag size (in bytes)
+    pub const AES_256_GCM_TAG_LEN: usize = 16;
+    /// AES256-XTS key length (in bytes)
+    pub const AES_256_XTS_KEY_LEN: usize = 64;
+    /// AES256-XTS tweak length (in bytes)
+    pub const AES_256_XTS_TWEAK_LEN: usize = 16;
+
+    /// Returns the tag length of the [`SymKeyType`] if it is an AEAD key
+    pub const fn tag_len(&self) -> Option<usize> {
+        match self {
+            SymKeyType::Aes256Gcm => Some(Self::AES_256_GCM_TAG_LEN),
+            SymKeyType::Aes256Xts => None,
+        }
+    }
+
+    /// Returns true if the [`SymKeyType`] is an AEAD key
+    pub const fn is_aead(&self) -> bool {
+        self.tag_len().is_some()
+    }
+}
+
+impl Display for SymKeyType {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let s = match self {
+            Self::Aes256Gcm => "AES-256-GCM",
+            Self::Aes256Xts => "AES-256-XTS",
+        };
+        write!(f, "{s}")
+    }
+}
+
+impl From<SymKeyType> for Nid {
+    fn from(value: SymKeyType) -> Self {
+        match value {
+            SymKeyType::Aes256Gcm => Self::AES_256_GCM,
+            SymKeyType::Aes256Xts => Self::AES_256_XTS,
+        }
+    }
+}
+
+/// The `enum_dispatch` macros needs at least one local trait to be implemented.
+#[allow(unused)]
+#[enum_dispatch(SymKey)]
+trait SymKeyTrait {}
+
 /// Types of symmetric keys
 #[non_exhaustive]
+#[enum_dispatch()]
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum SymKey {
-    /// AES 256 key (32 bytes)
-    Aes256(Aes256Key),
+    /// AES 256 GCM key (32 bytes)
+    Aes256(Aes256GcmKey),
+    /// AES 256 XTS key (64 bytes)
+    Aes256Xts(Aes256XtsKey),
 }
 
 impl SymKey {
@@ -53,7 +120,8 @@ impl SymKey {
     /// This function will return an error if the Key cannot be generated.
     pub fn random(key_tp: SymKeyType) -> Result<Self> {
         match key_tp {
-            SymKeyType::Aes256 => Ok(Self::Aes256(random_array().map(|v| v.into())?)),
+            SymKeyType::Aes256Gcm => Ok(Self::Aes256(random_array().map(|v| v.into())?)),
+            SymKeyType::Aes256Xts => Ok(Self::Aes256Xts(random_array().map(|v| v.into())?)),
         }
     }
 
@@ -61,13 +129,32 @@ impl SymKey {
     pub fn value(&self) -> &[u8] {
         match self {
             Self::Aes256(key) => key.value(),
+            Self::Aes256Xts(key) => key.value(),
         }
     }
-}
 
-impl From<Aes256Key> for SymKey {
-    fn from(value: Aes256Key) -> Self {
-        Self::Aes256(value)
+    /// Return the key type of this [`SymKey`].
+    pub fn key_type(&self) -> SymKeyType {
+        match self {
+            Self::Aes256(_) => SymKeyType::Aes256Gcm,
+            Self::Aes256Xts(_) => SymKeyType::Aes256Xts,
+        }
+    }
+
+    /// Try to create a symmetric key using the provided data.
+    ///
+    /// * `key_tp` - type of the symmetric key
+    /// * `data`   - raw key data
+    ///
+    /// # Errors
+    ///
+    /// This function will return an error if the key cannot be created, e.g.
+    /// because the provided data is too small or too large.
+    pub fn try_from_data(key_tp: SymKeyType, data: Confidential<Vec<u8>>) -> Result<Self> {
+        match key_tp {
+            SymKeyType::Aes256Gcm => Ok(Self::Aes256(data.try_into()?)),
+            SymKeyType::Aes256Xts => Ok(Self::Aes256Xts(data.try_into()?)),
+        }
     }
 }
 
@@ -96,12 +183,12 @@ pub(crate) fn hkdf_rfc_5869<const COUNT: usize>(
     Ok(res)
 }
 
-/// Derive a symmetric key from a private and a public key.
+/// Derive a symmetric AES 256 GCM key from a private and a public key.
 ///
 /// # Errors
 ///
 /// This function will return an error if something went bad in OpenSSL.
-pub(crate) fn derive_key(k1: &PKeyRef<Private>, k2: &PKeyRef<Public>) -> Result<Aes256Key> {
+pub fn derive_aes256_gcm_key(k1: &PKeyRef<Private>, k2: &PKeyRef<Public>) -> Result<Aes256GcmKey> {
     let mut der = Deriver::new(k1)?;
     der.set_peer(k2)?;
     let mut key = der.derive_to_vec()?;
@@ -109,7 +196,7 @@ pub(crate) fn derive_key(k1: &PKeyRef<Private>, k2: &PKeyRef<Public>) -> Result<
     let secr = Confidential::new(key);
 
     // Panic: does not panic as SHA256 digest is 32 bytes long
-    Ok(Aes256Key::new(
+    Ok(Aes256GcmKey::new(
         hash(MessageDigest::sha256(), secr.value())?
             .as_ref()
             .try_into()
@@ -122,49 +209,138 @@ pub(crate) fn derive_key(k1: &PKeyRef<Private>, k2: &PKeyRef<Public>) -> Result<
 /// # Errors
 ///
 /// This function will return an error if the entropy source fails or is not available.
-pub(crate) fn random_array<const COUNT: usize>() -> Result<[u8; COUNT]> {
+pub fn random_array<const COUNT: usize>() -> Result<[u8; COUNT]> {
     let mut rand = [0; COUNT];
     rand_bytes(&mut rand)?;
     Ok(rand)
 }
 
-/// Generate a new random EC-SECP521R1 key.
+/// Generate a new random EC key.
 ///
 /// # Errors
 ///
 /// This function will return an error if the key could not be generated by OpenSSL.
-pub(crate) fn gen_ec_key() -> Result<PKey<Private>> {
-    let group = EcGroup::from_curve_name(Nid::SECP521R1)?;
+pub fn gen_ec_key(nid: Nid) -> Result<PKey<Private>> {
+    let group = EcGroup::from_curve_name(nid)?;
     let key: EcKey<Private> = EcKey::generate(&group)?;
     PKey::from_ec_key(key).map_err(Error::Crypto)
 }
 
 /// Result type for an AES encryption in GCM mode..
-#[derive(Debug)]
-pub(crate) struct AesGcmResult {
+#[derive(PartialEq, Eq, Debug)]
+pub struct AeadEncryptionResult {
     /// The result.
     ///
     /// [`Vec<u8>`] with the following content:
     /// 1. `aad`
     /// 2. `encr(conf)`
     /// 3. `aes gcm tag`
-    pub buf: Vec<u8>,
+    pub(crate) buf: Vec<u8>,
     /// The position of the authenticated data in [`Self::buf`]
-    pub aad_range: Range<usize>,
+    pub(crate) aad_range: Range<usize>,
     /// The position of the encrypted data in [`Self::buf`]
-    pub encr_range: Range<usize>,
+    pub(crate) encr_range: Range<usize>,
     /// The position of the tag in [`Self::buf`]
-    #[allow(unused)]
-    // here for completeness
-    pub tag_range: Range<usize>,
+    pub(crate) tag_range: Range<usize>,
 }
 
-impl AesGcmResult {
+/// Result type for an AES decryption in GCM mode..
+#[derive(PartialEq, Eq, Debug)]
+pub struct AeadDecryptionResult {
+    /// The result.
+    ///
+    /// [`Vec<u8>`] with the following content:
+    /// 1. `aad`
+    /// 2. `decr(conf)`
+    /// 3. `aes gcm tag`
+    buf: Confidential<Vec<u8>>,
+    /// The position of the authenticated data in [`Self::buf`]
+    aad_range: Range<usize>,
+    /// The position of the authenticated data in [`Self::buf`]
+    data_range: Range<usize>,
+    /// The position of the tag in [`Self::buf`]
+    tag_range: Range<usize>,
+}
+
+impl AeadEncryptionResult {
     /// Deconstruct the result to just the resulting data w/o ranges.
-    pub(crate) fn data(self) -> Vec<u8> {
+    pub fn into_buf(self) -> Vec<u8> {
         let Self { buf, .. } = self;
         buf
     }
+
+    /// Deconstruct the result into all parts: additional authenticated data,
+    /// cipher data, and tag.
+    #[allow(unused)]
+    // here for completeness
+    pub(crate) fn into_parts(self) -> (Vec<u8>, Vec<u8>, Vec<u8>) {
+        let Self {
+            buf,
+            aad_range,
+            encr_range,
+            tag_range,
+        } = self;
+
+        (
+            buf[aad_range].to_vec(),
+            buf[encr_range].to_vec(),
+            buf[tag_range].to_vec(),
+        )
+    }
+
+    /// Deconstruct the result to the resulting ciphered data w/o ranges.
+    #[allow(unused)]
+    // here for completeness
+    pub(crate) fn into_cipher(self) -> Vec<u8> {
+        let Self {
+            buf,
+            aad_range: _,
+            encr_range,
+            ..
+        } = self;
+
+        buf[encr_range].to_vec()
+    }
+}
+
+impl AeadDecryptionResult {
+    /// Deconstruct the result to just the resulting data w/o ranges.
+    pub fn into_buf(self) -> Confidential<Vec<u8>> {
+        let Self { buf, .. } = self;
+        buf
+    }
+
+    /// Deconstruct the result into all parts: additional data, plain data, and tag.
+    #[allow(unused)]
+    // here for completeness
+    pub(crate) fn into_parts(self) -> (Vec<u8>, Confidential<Vec<u8>>, Vec<u8>) {
+        let Self {
+            buf,
+            aad_range,
+            data_range,
+            tag_range,
+        } = self;
+
+        (
+            buf.value()[aad_range].to_vec(),
+            Confidential::new(buf.value()[data_range].to_vec()),
+            buf.value()[tag_range].to_vec(),
+        )
+    }
+
+    /// Deconstruct the result to the resulting data w/o ranges.
+    #[allow(unused)]
+    // here for completeness
+    pub(crate) fn into_plain(self) -> Confidential<Vec<u8>> {
+        let Self {
+            buf,
+            aad_range: _,
+            data_range,
+            ..
+        } = self;
+
+        Confidential::new(buf.value()[data_range].to_vec())
+    }
 }
 
 /// Encrypt confidential Data with a symmetric key and provida a gcm tag.
@@ -173,27 +349,23 @@ impl AesGcmResult {
 /// * `iv` - initialisation vector
 /// * `aad` - additional authentic data
 /// * `conf` - data to be encrypted
+/// * `tag_len` - length of the authentication tag to generate (in bytes)
 ///
 /// # Errors
 ///
 /// This function will return an error if the data could not be encrypted by OpenSSL.
-pub(crate) fn encrypt_aes_gcm(
+pub fn encrypt_aead(
     key: &SymKey,
     iv: &[u8],
     aad: &[u8],
     conf: &[u8],
-) -> Result<AesGcmResult> {
-    let mut tag = vec![0xff; AES_256_GCM_TAG_SIZE];
-    let encr = match key {
-        SymKey::Aes256(key) => encrypt_aead(
-            Cipher::aes_256_gcm(),
-            key.value(),
-            Some(iv),
-            aad,
-            conf,
-            &mut tag,
-        )?,
-    };
+) -> Result<AeadEncryptionResult> {
+    let tag_len = key.key_type().tag_len().ok_or_else(|| Error::NoAeadKey)?;
+
+    let nid = key.key_type().into();
+    let cipher = Cipher::from_nid(nid).ok_or(Error::UnsupportedCipher(nid))?;
+    let mut tag = vec![0x0u8; tag_len];
+    let encr = openssl_encrypt_aead(cipher, key.value(), Some(iv), aad, conf, &mut tag)?;
 
     let mut buf = vec![0; aad.len() + encr.len() + tag.len()];
     let aad_range = Range {
@@ -212,7 +384,7 @@ pub(crate) fn encrypt_aes_gcm(
     buf[aad_range.clone()].copy_from_slice(aad);
     buf[encr_range.clone()].copy_from_slice(&encr);
     buf[tag_range.clone()].copy_from_slice(&tag);
-    Ok(AesGcmResult {
+    Ok(AeadEncryptionResult {
         buf,
         aad_range,
         encr_range,
@@ -234,28 +406,53 @@ pub(crate) fn encrypt_aes_gcm(
 /// # Errors
 ///
 /// This function will return an error if the data could not be encrypted by OpenSSL.
-pub(crate) fn decrypt_aes_gcm(
+pub fn decrypt_aead(
     key: &SymKey,
     iv: &[u8],
     aad: &[u8],
     encr: &[u8],
     tag: &[u8],
-) -> Result<Confidential<Vec<u8>>> {
-    let decr = match key {
-        SymKey::Aes256(key) => {
-            decrypt_aead(Cipher::aes_256_gcm(), key.value(), Some(iv), aad, encr, tag)
-        }
-    }
-    .map_err(|ssl_err| {
-        // Empty error-stack -> no internal ssl error but decryption failed.
-        // Very likely due to a tag mismatch.
-        if ssl_err.errors().is_empty() {
-            Error::GcmTagMismatch
-        } else {
-            Error::Crypto(ssl_err)
-        }
-    })?;
-    Ok(decr.into())
+) -> Result<AeadDecryptionResult> {
+    match key {
+        SymKey::Aes256(_) => {}
+        SymKey::Aes256Xts(_) => return Err(Error::NoAeadKey),
+    };
+    let nid = key.key_type().into();
+    let cipher = Cipher::from_nid(nid).ok_or(Error::UnsupportedCipher(nid))?;
+    let decr =
+        openssl_decrypt_aead(cipher, key.value(), Some(iv), aad, encr, tag).map_err(|ssl_err| {
+            // Empty error-stack -> no internal ssl error but decryption failed.
+            // Very likely due to a tag mismatch.
+            if ssl_err.errors().is_empty() {
+                Error::GcmTagMismatch
+            } else {
+                Error::Crypto(ssl_err)
+            }
+        })?;
+    let mut conf = Confidential::new(vec![0; aad.len() + decr.len() + tag.len()]);
+    let aad_range = Range {
+        start: 0,
+        end: aad.len(),
+    };
+    let data_range = Range {
+        start: aad.len(),
+        end: aad.len() + decr.len(),
+    };
+    let tag_range = Range {
+        start: aad.len() + decr.len(),
+        end: aad.len() + decr.len() + tag.len(),
+    };
+
+    let buf = conf.value_mut();
+    buf[aad_range.clone()].copy_from_slice(aad);
+    buf[data_range.clone()].copy_from_slice(&decr);
+    buf[tag_range.clone()].copy_from_slice(tag);
+    Ok(AeadDecryptionResult {
+        buf: conf,
+        aad_range,
+        data_range,
+        tag_range,
+    })
 }
 
 /// Calculate the hash of a slice.
@@ -342,7 +539,7 @@ pub(crate) fn verify_signature<T: HasPublic>(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{get_test_asset, test_utils::*};
+    use crate::{get_test_asset, test_utils::*, PvCoreError};
 
     #[test]
     fn sign_ec() {
@@ -350,7 +547,7 @@ mod tests {
 
         let data = "sample".as_bytes();
         let sign = sign_msg(&ec_key, MessageDigest::sha512(), data).unwrap();
-        assert!(sign.len() <= 139 && sign.len() >= 137);
+        assert!(sign.len() <= 139, "value is: {}", sign.len());
 
         assert!(verify_signature(&ec_key, MessageDigest::sha512(), data, &sign).unwrap());
     }
@@ -380,17 +577,17 @@ mod tests {
     }
 
     #[test]
-    fn derive_key() {
+    fn derive_aes256_gcm_key() {
         let (cust_key, host_key) = get_test_keys();
 
-        let exp_key: Aes256Key = [
+        let exp_key: Aes256GcmKey = [
             0x75, 0x32, 0x77, 0x55, 0x8f, 0x3b, 0x60, 0x3, 0x41, 0x9e, 0xf2, 0x49, 0xae, 0x3c,
             0x4b, 0x55, 0xaa, 0xd7, 0x7d, 0x9, 0xd9, 0x7f, 0xdd, 0x1f, 0xc8, 0x8f, 0xd8, 0xf0,
             0xcf, 0x22, 0xf1, 0x49,
         ]
         .into();
 
-        let calc_key = super::derive_key(&cust_key, &host_key).unwrap();
+        let calc_key = super::derive_aes256_gcm_key(&cust_key, &host_key).unwrap();
 
         assert_eq!(&calc_key, &exp_key);
     }
@@ -432,23 +629,26 @@ mod tests {
             0x4d, 0x23, 0xc3, 0xce, 0xc3, 0x34, 0xb4, 0x9b, 0xdb, 0x37, 0x0c, 0x43, 0x7f, 0xec,
             0x78, 0xde,
         ];
-        let aes_gcm_res = vec![
-            0x4d, 0x23, 0xc3, 0xce, 0xc3, 0x34, 0xb4, 0x9b, 0xdb, 0x37, 0x0c, 0x43, 0x7f, 0xec,
-            0x78, 0xde, 0xf7, 0x26, 0x44, 0x13, 0xa8, 0x4c, 0x0e, 0x7c, 0xd5, 0x36, 0x86, 0x7e,
-            0xb9, 0xf2, 0x17, 0x36, 0x67, 0xba, 0x05, 0x10, 0x26, 0x2a, 0xe4, 0x87, 0xd7, 0x37,
-            0xee, 0x62, 0x98, 0xf7, 0x7e, 0x0c,
+        let aes_gcm_ciphertext = [
+            0xf7, 0x26, 0x44, 0x13, 0xa8, 0x4c, 0x0e, 0x7c, 0xd5, 0x36, 0x86, 0x7e, 0xb9, 0xf2,
+            0x17, 0x36,
         ];
+        let aes_gcm_tag = [
+            0x67, 0xba, 0x05, 0x10, 0x26, 0x2a, 0xe4, 0x87, 0xd7, 0x37, 0xee, 0x62, 0x98, 0xf7,
+            0x7e, 0x0c,
+        ];
+        let aes_gcm_res = [aes_gcm_aad, aes_gcm_ciphertext, aes_gcm_tag].concat();
         let key = SymKey::Aes256(aes_gcm_key.into());
 
-        let AesGcmResult {
+        let AeadEncryptionResult {
             buf,
             aad_range,
             encr_range,
             tag_range,
-        } = encrypt_aes_gcm(&key, &aes_gcm_iv, &aes_gcm_aad, aes_gcm_plain.value()).unwrap();
+        } = encrypt_aead(&key, &aes_gcm_iv, &aes_gcm_aad, aes_gcm_plain.value()).unwrap();
         assert_eq!(buf, aes_gcm_res);
 
-        let conf = decrypt_aes_gcm(
+        let conf = decrypt_aead(
             &key,
             &aes_gcm_iv,
             &buf[aad_range],
@@ -456,7 +656,42 @@ mod tests {
             &buf[tag_range],
         )
         .unwrap();
-        assert_eq!(conf, aes_gcm_plain);
+        assert_eq!(&conf.buf.value()[conf.aad_range], &aes_gcm_aad);
+        assert_eq!(&conf.buf.value()[conf.data_range], aes_gcm_plain.value());
+        assert_eq!(&conf.buf.value()[conf.tag_range], &aes_gcm_tag);
+
+        let (aad, ciphertext, tag) =
+            encrypt_aead(&key, &aes_gcm_iv, &aes_gcm_aad, aes_gcm_plain.value())
+                .unwrap()
+                .into_parts();
+        assert_eq!(aes_gcm_aad, aad.as_slice());
+        assert_eq!(aes_gcm_ciphertext, ciphertext.as_slice());
+        assert_eq!(aes_gcm_tag, tag.as_slice());
+
+        let (aad2, plaintext, tag2) = decrypt_aead(&key, &aes_gcm_iv, &aad, &ciphertext, &tag)
+            .unwrap()
+            .into_parts();
+        assert_eq!(aes_gcm_aad, aad2.as_slice());
+        assert_eq!(aes_gcm_plain, plaintext);
+        assert_eq!(aes_gcm_tag, tag2.as_slice());
+    }
+
+    #[test]
+    fn aes_gcm_fails_wrong_keytype() {
+        let aes_gcm_iv = [
+            0x99, 0xaa, 0x3e, 0x68, 0xed, 0x81, 0x73, 0xa0, 0xee, 0xd0, 0x66, 0x84,
+        ];
+        let aes_gcm_plain = Confidential::new(vec![
+            0xf5, 0x6e, 0x87, 0x05, 0x5b, 0xc3, 0x2d, 0x0e, 0xeb, 0x31, 0xb2, 0xea, 0xcc, 0x2b,
+            0xf2, 0xa5,
+        ]);
+        let aes_gcm_aad = [
+            0x4d, 0x23, 0xc3, 0xce, 0xc3, 0x34, 0xb4, 0x9b, 0xdb, 0x37, 0x0c, 0x43, 0x7f, 0xec,
+            0x78, 0xde,
+        ];
+
+        let key = SymKey::random(SymKeyType::Aes256Xts).unwrap();
+        encrypt_aead(&key, &aes_gcm_iv, &aes_gcm_aad, aes_gcm_plain.value()).expect_err("");
     }
 
     #[test]
@@ -479,4 +714,55 @@ mod tests {
 
         assert_eq!(hmac, exp);
     }
+
+    #[test]
+    fn from_symkeytype() {
+        assert_eq!(
+            <SymKeyType as Into<Nid>>::into(SymKeyType::Aes256Gcm),
+            Nid::AES_256_GCM
+        );
+        assert_eq!(
+            <SymKeyType as Into<Nid>>::into(SymKeyType::Aes256Xts),
+            Nid::AES_256_XTS
+        );
+    }
+
+    #[test]
+    fn key_type() {
+        assert_eq!(
+            SymKey::random(SymKeyType::Aes256Gcm).unwrap().key_type(),
+            SymKeyType::Aes256Gcm
+        );
+        assert_eq!(
+            SymKey::random(SymKeyType::Aes256Xts).unwrap().key_type(),
+            SymKeyType::Aes256Xts
+        );
+    }
+
+    #[test]
+    fn try_from_and_into() {
+        let data = [0x1u8; 32];
+        let key: SymKey = Aes256GcmKey::new(data).into();
+        assert_eq!(key.value(), &data);
+        let key_aes: Aes256GcmKey = key.try_into().expect("should not fail");
+        assert_eq!(key_aes.value(), &data);
+    }
+
+    #[test]
+    fn try_from_data() {
+        let data = [0x3u8; 32];
+        let key = SymKey::try_from_data(SymKeyType::Aes256Gcm, Confidential::new(data.into()))
+            .expect("should not fail");
+        assert_eq!(&data, key.value());
+        let key_aes: Aes256GcmKey = key.try_into().expect("should not fail");
+        assert_eq!(&data, key_aes.value());
+
+        assert!(matches!(
+            SymKey::try_from_data(SymKeyType::Aes256Gcm, Confidential::new([0x4u8; 33].into())),
+            Err(Error::PvCore(PvCoreError::LengthMismatch {
+                expected: 32,
+                actual: 33
+            }))
+        ));
+    }
 }
