@@ -36,6 +36,13 @@ pub enum Error {
         source: std::io::Error,
     },
 
+    #[error("Cannot rename '{src}' to '{dst}'")]
+    FileAccessRename {
+        src: String,
+        dst: String,
+        source: std::io::Error,
+    },
+
     #[error("Cannot encode secrets (Too many secrets)")]
     ManySecrets,
 
@@ -68,6 +75,15 @@ pub enum Error {
     Io(#[from] std::io::Error),
     #[error(transparent)]
     ParseInt(#[from] std::num::ParseIntError),
+
+    #[error("Cannot decode hex string: Size {0} is not a multiple of two")]
+    InvHexStringSize(usize),
+
+    #[error("Cannot decode hex string")]
+    InvHexStringChar { source: std::num::ParseIntError },
+
+    #[error("Expected size {expected}, actual {actual}")]
+    LengthMismatch { expected: usize, actual: usize },
 }
 
 /// Error cases for I/O operations
