@@ -1,15 +1,8 @@
 // SPDX-License-Identifier: MIT
 //
 // Copyright IBM Corp. 2023, 2024
-#![deny(missing_docs)]
-#![warn(
-    missing_debug_implementations,
-    trivial_numeric_casts,
-    unstable_features,
-    unused_import_braces,
-    unused_qualifications
-)]
 #![doc = include_str!("../README.md")]
+mod confidential;
 mod error;
 mod macros;
 mod utils;
@@ -28,7 +21,7 @@ pub mod attest {
 pub mod misc {
     pub use crate::utils::pv_guest_bit_set;
     pub use crate::utils::{create_file, open_file, read_exact_file, read_file, write_file};
-    pub use crate::utils::{parse_hex, to_u16, to_u32, try_parse_u128, try_parse_u64};
+    pub use crate::utils::{decode_hex, parse_hex, to_u16, to_u32, try_parse_u128, try_parse_u64};
     pub use crate::utils::{read, write};
     pub use crate::utils::{Flags, Lsb0Flags64, Msb0Flags64};
 }
@@ -46,6 +39,7 @@ pub mod uv {
 
 /// Functionalities to verify UV requests
 pub mod request {
+    pub use crate::confidential::{Confidential, Zeroize};
     /// Version number of the request in system endianness
     pub type RequestVersion = u32;
     /// Request magic value
