@@ -2,12 +2,14 @@
 //
 // Copyright IBM Corp. 2023, 2024
 
+use std::path::Path;
+
 use crate::cli::{AddSecretType, CreateSecretFlags, CreateSecretOpt};
 use anyhow::{anyhow, bail, Context, Error, Result};
 use log::{debug, info, trace, warn};
 use pv::{
     misc::{
-        open_file, parse_hex, pv_guest_bit_set, read_exact_file, read_file, try_parse_u128,
+        decode_hex, open_file, pv_guest_bit_set, read_exact_file, read_file, try_parse_u128,
         try_parse_u64, write,
     },
     request::{
@@ -20,8 +22,12 @@ use pv::{
 use serde_yaml::Value;
 use utils::get_writer_from_cli_file_arg;
 
-fn write_out<D: AsRef<[u8]>>(path: &str, data: D, ctx: &str) -> pv::Result<()> {
-    let mut wr = get_writer_from_cli_file_arg(path)?;
+fn write_out<P, D>(path: &P, data: D, ctx: &str) -> pv::Result<()>
+where
+    P: AsRef<Path>,
+    D: AsRef<[u8]>,
+{
+    let mut wr = get_writer_from_cli_file_arg(path.as_ref())?;
     write(&mut wr, data, path, ctx)?;
     Ok(())
 }
@@ -49,13 +55,14 @@ pub fn create(opt: &CreateSecretOpt) -> Result<()> {
     debug!("Added all host-keys");
 
     // build + encrypt the request
-    let rq = ReqEncrCtx::random(SymKeyType::Aes256).context("Failed to generate random input")?;
+    let rq =
+        ReqEncrCtx::random(SymKeyType::Aes256Gcm).context("Failed to generate random input")?;
     let ser_asrbc = asrcb.encrypt(&rq)?;
     warn!("Successfully generated the request");
     write_out(&opt.output, ser_asrbc, "add-secret request")?;
     info!("Successfully wrote the request to '{}'", &opt.output);
 
-    write_secret(&opt.secret, &asrcb)
+    write_secret(&opt.secret, &asrcb, &opt.output)
 }
 
 /// Read+parse the first key from the buffer.
@@ -141,7 +148,7 @@ fn build_asrcb(opt: &CreateSecretOpt) -> Result<AddSecretRequest> {
 // Try to extract a Config-UId from a yaml structure
 // The cuid field can be embedded in an abritray amount of Mappings
 // The function takes the first cuid it founds (width search).
-fn try_from_val(val: Value) -> anyhow::Result<ConfigUid> {
+fn try_from_val(val: Value) -> Result<ConfigUid> {
     fn get_cuid_from_mapping(val: &Value, depth: u8) -> Option<String> {
         if depth >= 8 {
             return None;
@@ -174,7 +181,7 @@ fn try_from_val(val: Value) -> anyhow::Result<ConfigUid> {
     if cuid.len() != ::std::mem::size_of::<ConfigUid>() * 2 {
         return Err(anyhow!(format!("len invalid ({})", cuid.len())));
     }
-    let cuid: ConfigUid = parse_hex(&cuid)
+    let cuid: ConfigUid = decode_hex(&cuid)?
         .try_into()
         .map_err(|_| anyhow!("Cannot parse hex number".to_string()))?;
     Ok(cuid)
@@ -200,7 +207,11 @@ fn read_cuid(asrcb: &mut AddSecretRequest, opt: &CreateSecretOpt) -> Result<()>
 }
 
 /// Write the generated secret (if any) to the specified output stream
-fn write_secret(secret: &AddSecretType, asrcb: &AddSecretRequest) -> Result<()> {
+fn write_secret<P: AsRef<Path>>(
+    secret: &AddSecretType,
+    asrcb: &AddSecretRequest,
+    outp_path: P,
+) -> Result<()> {
     if let AddSecretType::Association {
         name,
         stdout,
@@ -208,23 +219,31 @@ fn write_secret(secret: &AddSecretType, asrcb: &AddSecretRequest) -> Result<()>
         ..
     } = secret
     {
-        let gen_path: String = name
+        let gen_name: String = name
             .chars()
             .map(|c| if c.is_whitespace() { '_' } else { c })
             .collect();
+        let mut gen_path = outp_path
+            .as_ref()
+            .parent()
+            .with_context(|| format!("Cannot open directory of {:?}", outp_path.as_ref()))?
+            .to_owned();
+        gen_path.push(format!("{gen_name}.yaml"));
 
         // write non confidential data (=name+id) to a yaml
         let secret_info = serde_yaml::to_string(asrcb.guest_secret())?;
         if stdout.to_owned() {
             println!("{secret_info}");
         } else {
-            let secret_info_path = format!("{gen_path}.yaml");
-            write_out(&secret_info_path, secret_info, "association secret info")?;
+            write_out(&gen_path, secret_info, "association secret info")?;
             debug!(
                 "Non-confidential secret information: {:x?}",
                 asrcb.guest_secret()
             );
-            warn!("Successfully wrote association info to '{secret_info_path}'");
+            warn!(
+                "Successfully wrote association info to '{}'",
+                gen_path.display()
+            );
         }
 
         if let Some(path) = secret_out {
